local timer = LibStub("AceTimer-3.0")
local encounters = HudMap:GetModule("Encounters")
encounters:RegisterEvent("UNIT_TARGET", "Yell")  -- the method is named Yell, wired but can be used to all events. Defined is in Addons/HudMap/modules/encounters.lua
encounters:RegisterEvent("PLAYER_REGEN_ENABLED", "Yell")

local HudMap = HudMap
local L = LibStub("AceLocale-3.0"):GetLocale("HudMap")
local free = HudMap.free
local TormentedMarkers = {}
local CountdownMarkers = {}
local checknow
local GloryFire = GetSpellInfo(99252)


local function IsTank(unit)
	if UnitGroupRolesAssigned(unit) == "TANK" then  -- Please Ensure every person is assigned a right role
		return true
	elseif UnitAura(unit, GloryFire) then
		return true
	end
end


local function register(e)	
	encounters:RegisterEncounterMarker(e)
	return e
end

local function checkwipe()
	local flag = true
	SetMapToCurrentZone()
	
	
	for i=1, 40 do
		local unit = "raid" .. i
		if UnitIsVisible(unit) and UnitAffectingCombat(unit) then
			flag = false
			break
		end
	end
	
	return flag
end


local Baleroc = {
	name = L["Baleroc"],
	startEncounterIDs = 53494,--, --!!
	endEncounterIDs = 53494,--,
	---[[
	options = {
		Tormented = L["Tormented"],
		ShardsTorment = L["ShardsTorment"],
		Countdown = L["Countdown"],
	},
	defaults = {
		Tormented = true,
		TormentedColor = {r = 0.7, g = 0.7, b = 0.7, a = 0.5},
		ShardsTorment = true,
		ShardsTormentColor = {r = 0.6, g = 0.2, b = 0.8, a = 0.5},-- here
		Countdown = true,
		CountdownColor = {r = 0.93, g = 0.93, b = 0, a = 0.5},
	},
	Start = function(self)
		HudMap:Toggle(true)
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 99404 and  self:Option("TormentedEnabled") then
			local r,g,b,a = self:Option("TormentedColor")
			TormentedMarkers[destName] = register(HudMap:PlaceRangeMarkerOnPartyMember("highlight",destName,5,nil,r,g,b,a):Appear():SetLabel(destName))
		elseif spellID == 99516 and self:Option("CountdownEnabled") then
			local r,g,b,a = self:Option("CountdownColor")
			CountdownMarkers[destName] = register(HudMap:PlaceRangeMarkerOnPartyMember("highlight",destName,4,nil,r,g,b,a):Appear():SetLabel(destName))
		end
	end,
	PLAYER_REGEN_ENABLED = function(self)
		encounters:FreeEncounterMarkers()
		wipe(TormentedMarkers)
		wipe(CountdownMarkers)
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 99404 and TormentedMarkers[destName] then
			TormentedMarkers[destName] = free(TormentedMarkers[destName])
		elseif spellID == 99516 and CountdownMarkers[destName] then
			CountdownMarkers[destName] = free(CountdownMarkers[destName])
		end
	end,
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 99259 then
			checknow = true
			encounters:Delay(function()checknow = false end, 5)
			--[[debug
			print("checknow ѱֵΪtrue")
			--debug]]
		end
	end,
	UNIT_TARGET = function(self,unit)
		--[[debug
		if unit == "boss1" then
			print(checknow)
			print(UnitName("boss1target"))
			print(IsTank("boss1target"))
		end
		
		--@@debug]]
		if checknow then
			if unit == "boss1" and (not IsTank("boss1target")) then
				local r,g,b,a = self:Option("ShardsTormentColor")
				register(HudMap:PlaceStaticMarkerOnPartyMember("highlight", "boss1target", 4, 30,r,g,b,a):Appear())
			end
		end
	end,
}

encounters:RegisterModule(L["FireLand"], Baleroc)